import sys
import socket
import errno
from PyQt5.QtWidgets import QApplication, QMainWindow, QVBoxLayout, QWidget, QPushButton
from PyQt5.QtCore import pyqtSlot, QThread, pyqtSignal, Qt
from PyQt5.QtWidgets import QTextEdit
from PyQt5.QtGui import QKeySequence


class ReceiverThread(QThread):
    received_signal = pyqtSignal(str)
    connection_closed_signal = pyqtSignal()

    def __init__(self, client_socket, header_length):
        super().__init__()
        self.client_socket = client_socket
        self.header_length = header_length
        self.running = True
        self.last_username = None

    def run(self):
        while self.running:
            try:
                while True:
                    message_header = self.client_socket.recv(self.header_length)
                    if not len(message_header):
                        if self.running:
                            print("Connection closed by the server")
                        self.running = False
                        self.connection_closed_signal.emit()
                        self.terminate()
                        return
                    message_length = int(message_header.decode("utf-8").strip())
                    message = self.client_socket.recv(message_length).decode("utf-8")
                    self.received_signal.emit(message)
            except IOError as e:
                if e.errno != errno.EAGAIN and e.errno != errno.EWOULDBLOCK:
                    print("Reading error", str(e))
                    self.running = False
                    self.terminate()
                    return
                continue
            except Exception as e:
                print("General error", str(e))
                self.running = False
                self.terminate()
                return

    def stop(self):
        self.running = False
        self.wait()


class CustomTextEdit(QTextEdit):
    def __init__(self, parent=None, chat_client=None):
        super().__init__(parent)
        self.chat_client = chat_client
        self.setFixedHeight(90)

    def keyPressEvent(self, event):
        if event.key() == Qt.Key_Return or event.key() == Qt.Key_Enter:
            if event.modifiers() & Qt.ShiftModifier:
                self.insertPlainText("\n")
            else:
                if self.chat_client:
                    self.chat_client.write()
        elif event.matches(QKeySequence.Paste):
            clipboard_text = QApplication.clipboard().text()
            self.insertPlainText(clipboard_text)
        else:
            super().keyPressEvent(event)


class ChatClient(QMainWindow):
    HEADER_LENGTH = 10

    def __init__(self, ip, port, username):
        super().__init__()
        self.ip = ip
        self.port = port
        self.username = username
        self.client_socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.client_socket.connect((self.ip, self.port))
        self.client_socket.setblocking(False)
        self.send_username()
        self.initUI()
        self.receiver_thread = ReceiverThread(self.client_socket, self.HEADER_LENGTH)
        self.receiver_thread.received_signal.connect(self.update_text_area)
        self.receiver_thread.connection_closed_signal.connect(self.on_connection_closed)
        self.receiver_thread.start()

    def send_username(self):
        username_encoded = self.username.encode("utf-8")
        username_header = f"{len(username_encoded):<{self.HEADER_LENGTH}}".encode("utf-8")
        self.client_socket.send(username_header + username_encoded)

    def initUI(self):
        self.setWindowTitle(f"{self.username}'s Chat Client")
        self.setGeometry(100, 100, 400, 600)
        layout = QVBoxLayout()
        self.text_area = QTextEdit(self)
        self.text_area.setReadOnly(True)
        layout.addWidget(self.text_area)
        self.input_area = CustomTextEdit(self, chat_client=self)
        self.input_area.setPlaceholderText(f"Message as {self.username}")
        layout.addWidget(self.input_area)
        self.send_button = QPushButton("Send", self)
        self.send_button.clicked.connect(self.write)
        layout.addWidget(self.send_button)
        central_widget = QWidget()
        central_widget.setLayout(layout)
        self.setCentralWidget(central_widget)

    @pyqtSlot()
    def write(self):
        message = self.input_area.toPlainText()
        self.send_message(message)
        self.update_text_area(f"{self.username} > {message}")
        self.input_area.clear()

    def send_message(self, message):
        message_encoded = message.encode("utf-8")
        message_header = f"{len(message_encoded):<{self.HEADER_LENGTH}}".encode("utf-8")
        self.client_socket.send(message_header + message_encoded)

    @pyqtSlot(str)
    def update_text_area(self, message):
        username, _, _ = message.partition(' > ')
        if self.receiver_thread.last_username and (username != self.receiver_thread.last_username or
                                                   ("AI" in username and "AI" in self.receiver_thread.last_username)):
            self.text_area.append("")
        self.text_area.append(message.replace('\n', '<br>'))
        self.receiver_thread.last_username = username

    def on_connection_closed(self):
        self.close()

    def closeEvent(self, event):
        if self.receiver_thread.isRunning():
            self.receiver_thread.stop()
            self.client_socket.close()
        event.accept()

def main():
    app = QApplication(sys.argv)
    username = input("Enter your username: ")
    server_ip = input("Enter server IP (for default 127.0.0.1, press enter): ") or "127.0.0.1"
    server_port = input("Enter server port (for default 1234, press enter): ") or "1234"
    server_port = int(server_port)
    client = ChatClient(server_ip, server_port, username)
    print("Chat client is running...")
    client.show()
    sys.exit(app.exec_())

if __name__ == '__main__':
    app = QApplication(sys.argv)
    username = input("Enter your username: ")
    client = ChatClient("127.0.0.1", 1234, username)
    client.show()
    sys.exit(app.exec_())



import threading
import time
import os
from server_client.server import ChatServer
from server_client.ai_client import AIChatClient


def load_env_variables(filename='.env'):
    try:
        with open(filename) as f:
            for line in f:
                if line.strip() and not line.startswith('#'):
                    key, value = line.strip().split('=', 1)
                    os.environ[key] = value
    except FileNotFoundError:
        pass


def start_server(ip, port):
    server = ChatServer(ip, port)
    server.run()


def main(server_ip_address="127.0.0.1", server_port=1234, send_full_chat_history=True, ai_mode1_active=True,
         ai_mode1_interval=1, ai_mode1_model="gpt-3.5-turbo", ai_mode2_active=True, ai_mode2_interval=60,
         ai_mode2_model="gpt-3.5-turbo", ai_mode2_content="Say something interesting from a random Wikipedia page and"
                                                       " start your response with 'Did you know', but don't mention the"
                                                       " source."):
    load_env_variables()

    server_ip_address = os.getenv("SERVER_IP_ADDRESS", server_ip_address)
    server_port = int(os.getenv("SERVER_PORT", server_port))
    send_full_chat_history = os.getenv("SEND_FULL_CHAT_HISTORY", str(send_full_chat_history)) == "True"
    ai_mode1_active = os.getenv("AI_MODE1_ACTIVE", str(ai_mode1_active)) == "True"
    ai_mode1_interval = int(os.getenv("AI_MODE1_INTERVAL", ai_mode1_interval))
    ai_mode1_model = os.getenv("AI_MODE1_MODEL", ai_mode1_model)
    ai_mode2_active = os.getenv("AI_MODE2_ACTIVE", str(ai_mode2_active)) == "True"
    ai_mode2_interval = int(os.getenv("AI_MODE2_INTERVAL", ai_mode2_interval))
    ai_mode2_model = os.getenv("AI_MODE2_MODEL", ai_mode2_model)
    ai_mode2_content = os.getenv("AI_MODE2_CONTENT", ai_mode2_content)

    server = ChatServer(server_ip_address, server_port)
    server_thread = threading.Thread(target=server.run)
    server_thread.start()
    print("AI chat server is running...")
    time.sleep(1)

    if ai_mode1_active or ai_mode2_active:
        username = "AI"
        ai_client = AIChatClient(server, server_ip_address, server_port, username, ai_mode1_active, ai_mode1_interval,
                                 ai_mode1_model, ai_mode2_active, ai_mode2_interval, ai_mode2_model,
                                 send_full_chat_history, ai_mode2_content)
        threading.Thread(target=ai_client.receive_message).start()

    try:
        server_thread.join()
    except KeyboardInterrupt:
        print("Shutting down server and AI clients...")
        server.shutdown()
        if ai_client:
            ai_client.shutdown()
        server_thread.join()


if __name__ == '__main__':
    server_ip_address = "127.0.0.1"
    server_port = 1234

    os.environ['OPENAI_API_KEY'] = "<OPENAI_API_KEY_HERE>"
    send_full_chat_history = True
    ai_mode1_active = True
    ai_mode1_interval = 1
    ai_mode1_model = "gpt-3.5-turbo"
    ai_mode2_active = True
    ai_mode2_interval = 60
    ai_mode2_model = "gpt-3.5-turbo"
    ai_mode2_content= ("Say something interesting from a random Wikipedia page and start your response with"
                       " 'Did you know', but don't mention the source.")

    main(server_ip_address, server_port, send_full_chat_history, ai_mode1_active, ai_mode1_interval, ai_mode1_model,
         ai_mode2_active, ai_mode2_interval, ai_mode2_model, ai_mode2_content)



import os
import socket
import time
import openai
import errno


class AIChatClient:
    HEADER_LENGTH = 10

    def __init__(self, server, ip, port, username, mode1_enabled, mode1_interval, mode1_model, mode2_enabled,
                 mode2_interval, mode2_model, send_full_chat_history, mode2_content):
        self.ip = ip
        self.port = port
        self.username = username
        self.mode1_enabled = mode1_enabled
        self.mode1_interval = mode1_interval
        self.mode1_model = mode1_model
        self.mode2_enabled = mode2_enabled
        self.mode2_interval = mode2_interval
        self.mode2_model = mode2_model
        self.conversation_history = []
        self.send_full_chat_history = send_full_chat_history
        self.mode2_content = mode2_content
        self.client_socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.client_socket.connect((self.ip, self.port))
        self.client_socket.setblocking(False)
        self.send_username()
        self.line_count = 0
        self.last_response_time = float('inf')
        self.openai_client = openai.OpenAI(api_key=os.environ.get("OPENAI_API_KEY"))
        self.error_reported = False
        self.running = True
        self.server = server

    def shutdown(self):
        self.running = False
        try:
            self.client_socket.close()
        except Exception as e:
            print(f"Error closing socket: {e}")

    def send_username(self):
        username_encoded = self.username.encode("utf-8")
        username_header = f"{len(username_encoded):<{self.HEADER_LENGTH}}".encode("utf-8")
        self.client_socket.send(username_header + username_encoded)

    def receive_message(self):
        while self.running:
            try:
                if self.mode2_enabled and self.server.get_user_client_count() > 0:
                    if self.last_response_time == float('inf'):
                        self.last_response_time = time.time()
                    elif time.time() - self.last_response_time >= self.mode2_interval:
                        self.respond_with_new_message()
                        self.last_response_time = time.time()
                elif self.mode2_enabled and self.server.get_user_client_count() == 0:
                    self.last_response_time = float('inf')

                message_header = self.client_socket.recv(self.HEADER_LENGTH)
                if not len(message_header):
                    print("Connection closed by the server")
                    break
                message_length = int(message_header.decode("utf-8").strip())
                message = self.client_socket.recv(message_length).decode("utf-8")
                self.handle_message(message)

            except IOError as e:
                if e.errno == errno.EBADF:
                    print("Client socket closed, stopping receive loop")
                    break
                elif e.errno not in (errno.EAGAIN, errno.EWOULDBLOCK):
                    print(f"Reading error: {e}")
                    break

            except Exception as e:
                print(f"Error receiving message: {e}")
                break

    def handle_message(self, message):
        username, _, content = message.partition(' > ')
        if username.strip() != self.username:
            self.conversation_history.append({"role": "user", "content": content})
            if self.mode1_enabled:
                self.line_count += 1
                if self.line_count >= self.mode1_interval:
                    self.respond_to_message()
                    self.line_count = 0

    def handle_error(self, error_message):
        if not self.error_reported:
            print(error_message)
            self.send_message(error_message)
            self.error_reported = True

    def respond_to_message(self):
        try:
            messages_to_send = self.conversation_history if self.send_full_chat_history else\
                self.conversation_history[-self.mode1_interval:]
            print("Sending messages to OpenAI:", messages_to_send)
            chat_completion = self.openai_client.chat.completions.create(
                messages=messages_to_send,
                model=self.mode1_model,
            )
            response_text = chat_completion.choices[0].message.content
            self.send_message(response_text)
            self.conversation_history.append({"role": "assistant", "content": response_text})
        except Exception as e:
            self.handle_error(f"Error calling OpenAI API: {e}")

    def respond_with_new_message(self):
        try:
            chat_completion = self.openai_client.chat.completions.create(
                messages=[
                    {"role": "user", "content": self.mode2_content}
                ],
                model=self.mode2_model,
            )
            response_text = chat_completion.choices[0].message.content
            self.conversation_history.append({"role": "system", "content": response_text})
            self.send_message(response_text)
        except Exception as e:
            self.handle_error(f"Error calling OpenAI API: {e}")

    def send_message(self, message):
        message_encoded = message.encode("utf-8")
        message_header = f"{len(message_encoded):<{self.HEADER_LENGTH}}".encode("utf-8")
        self.client_socket.send(message_header + message_encoded)



import socket
import select


class ChatServer:
    HEADER_LENGTH = 10

    def __init__(self, ip, port):
        self.ip = ip
        self.port = port
        self.server_socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.server_socket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        self.server_socket.bind((self.ip, self.port))
        self.server_socket.listen()
        self.sockets_list = [self.server_socket]
        self.clients = {}
        self.running = True

    def get_user_client_count(self):
        return sum(1 for client in self.clients.values() if client['data'].decode('utf-8').startswith('AI') is False)

    def receive_message(self, client_socket):
        try:
            message_header = client_socket.recv(self.HEADER_LENGTH)
            if not len(message_header):
                return False
            message_length = int(message_header.decode("utf-8").strip())
            return {"header": message_header, "data": client_socket.recv(message_length)}
        except:
            return False

    def run(self):
        while self.running:
            try:
                read_sockets, _, exception_sockets = select.select(self.sockets_list, [], self.sockets_list)
                for notified_socket in read_sockets:
                    if notified_socket == self.server_socket:
                        client_socket, client_address = self.server_socket.accept()
                        user = self.receive_message(client_socket)
                        if user is False:
                            continue
                        self.sockets_list.append(client_socket)
                        self.clients[client_socket] = user
                        print(f"Accepted new connection from {client_address[0]}:{client_address[1]} username:{user['data'].decode('utf-8')}")
                    else:
                        message = self.receive_message(notified_socket)
                        if message is False:
                            print(f"Closed connection from {self.clients[notified_socket]['data'].decode('utf-8')}")
                            self.sockets_list.remove(notified_socket)
                            del self.clients[notified_socket]
                            continue
                        user = self.clients[notified_socket]
                        print(f"Received message from {user['data'].decode('utf-8')}: {message['data'].decode('utf-8')}")
                        for client_socket in self.clients:
                            if client_socket != notified_socket:
                                combined_message = user['data'] + b' > ' + message['data']
                                combined_header = f"{len(combined_message):<{self.HEADER_LENGTH}}".encode("utf-8")
                                client_socket.send(combined_header + combined_message)

                for notified_socket in exception_sockets:
                    self.sockets_list.remove(notified_socket)
                    del self.clients[notified_socket]

            except OSError as e:
                if self.running:
                    print(f"Server accept error: {e}")
                else:
                    print("Server socket closed")
                    break
            except Exception as e:
                print(f"Unexpected error: {e}")

    def shutdown(self):
        self.running = False
        self.server_socket.close()


def main():
    server_ip = input("Enter server IP (for default 127.0.0.1, press enter): ") or "127.0.0.1"
    server_port = input("Enter server port (for default 1234, press enter): ") or "1234"
    server_port = int(server_port)
    server = ChatServer(server_ip, server_port)

    try:
        print("Chat server is running...")
        server.run()
    except KeyboardInterrupt:
        print("Shutting down server...")
        server.shutdown()


if __name__ == "__main__":
    server = ChatServer("127.0.0.1", 1234)
    server.run()



from setuptools import setup, find_packages

with open('README.md', 'r', encoding='utf-8') as fh:
    long_description = fh.read()

setup(
    name='socktalk',
    version='0.1.7',
    packages=find_packages(),
    project_urls={
        'Source': 'https://github.com/mdkmk/socktalk'
    },
    install_requires=[
        'PyQt5==5.15.10',
        'openai==1.14.2'
    ],
    entry_points={
        'console_scripts': [
            'ai_server=server_client.start_server_with_ai_client:main',
            'chat_server=server_client.server:main',
            'chat_client=server_client.client:main'
        ],
    },
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: Mozilla Public License 2.0 (MPL 2.0)',
        'Operating System :: OS Independent',
    ],
    python_requires='>=3.6',
    description='Socket-based AI chat server and multi-user chat client',
    long_description=long_description,
    long_description_content_type='text/markdown'
)
